<?php

/**
 * Copyright © 2017 Filoblu S.r.l. All rights reserved.
 */

namespace FiloBlu\GaMeasurementProtocol\Model\Plugin;

use Magento\Store\Model\ScopeInterface;
use Magento\Quote\Api\CartRepositoryInterface;

/**
 * Plugin that extends Shipping Information saves. Save Additional Ga session id to related quote.
 */
class AddressInformationToQuoteGaSessionPlugin {

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scopeConfiguration;

    /**
     * Sales quote repository
     *
     * @var \Magento\Quote\Api\CartRepositoryInterface
     */
    protected $quoteRepository;

    /**
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfiguration
     * @param \Magento\Quote\Api\CartRepositoryInterface $quoteRepository
     */
    public function __construct(
    \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfiguration, \Magento\Quote\Api\CartRepositoryInterface $quoteRepository
    ) {
        $this->scopeConfiguration = $scopeConfiguration;
        $this->quoteRepository = $quoteRepository;
    }

    /**
     * @param \Magento\Sales\Api\OrderManagementInterface $management
     * @param int $cartId
     * @param \Magento\Checkout\Api\Data\ShippingInformationInterface $addressInformation
     * @return array($cartId, $addressInformation)
     */
    public function beforeSaveAddressInformation(\Magento\Checkout\Api\ShippingInformationManagementInterface $management, $cartId, \Magento\Checkout\Api\Data\ShippingInformationInterface $addressInformation) {
        $gaSessionId = null;

        if ($addressInformation) {
            $address = $addressInformation->getShippingAddress();
            if ($address) {
                $extensionAttributes = $address->getExtensionAttributes();
                if ($extensionAttributes) {
                    $gaSessionId = $extensionAttributes->getGaSessionId();
                }
            }
        }

        if ($gaSessionId) {
            $quote = $this->quoteRepository->getActive($cartId);

            $quote->setData('ga_session_id', $gaSessionId);

            try {
                $this->quoteRepository->save($quote);
            } catch (\Exception $e) {
                $this->logger->critical($e);
                throw new InputException(__('Unable to save ga session ID.'));
            }
        }

        return array($cartId, $addressInformation);
    }

}
